pub enum Easing {
Linear,
InPowi(i32),
OutPowi(i32),
InOutPowi(i32),
InPowf(f64),
OutPowf(f64),
InOutPowf(f64),
}Expand description
Curves the motion of a Tween.
Variants
Linear
Maintains a constant speed for the duration of the Tween.
InPowi(i32)
Causes the Tween to start slow and speed up. A higher
value causes the Tween to speed up more dramatically.
OutPowi(i32)
Causes the Tween to start fast and slow down. A higher
value causes the Tween to slow down more dramatically.
InOutPowi(i32)
Causes the Tween to start slow, speed up, and then slow
back down. A higher values causes the Tween to have more
dramatic speed changes.
InPowf(f64)
Causes the Tween to start slow and speed up. A higher
value causes the Tween to speed up more dramatically.
This is similar to InPowi, but allows
for float intensity values at the cost of being more
CPU intensive.
OutPowf(f64)
Causes the Tween to start fast and slow down. A higher
value causes the Tween to slow down more dramatically.
This is similar to OutPowi, but allows
for float intensity values at the cost of being more
CPU intensive.
InOutPowf(f64)