pub struct ResourceLimitReached;Expand description
An error that is returned when a resource cannot be added because the maximum capacity for that resource has been reached.
You can adjust these capacities using Capacities.